
#include "hardware.h"
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include "ctapi.h"
#include "ctbcs.h"


#ifdef PARPORT

void open_door() {
  if (ioperm(BASEPORT, 3, 1)) {perror("ioperm"); exit(1);}
  outb(255, BASEPORT);
  sleep(OPENTIME);
  outb(0, BASEPORT);
  if (ioperm(BASEPORT, 3, 0)) {perror("ioperm"); exit(1);}
}

#endif

#ifdef SERIAL
 
void open_door() {
  int fd;
  fd = open(BASEPORT, O_RDWR | O_NOCTTY | O_NDELAY);
  if (fd == -1) {
     perror("open_port: Unable to open tty ");
  } else
     fcntl(fd, F_SETFL, 0);
  sleep(OPENTIME);
  close(fd);
}

#endif

void watchdog();


#ifdef EASY_READ

bool checksumm(char *code);

string read_key() {
  int STOP=FALSE;
  int fd,c, res;
  struct termios oldtio,newtio;
  char buf[255]="";

  fd = open(DEVICE, O_RDWR | O_NOCTTY );
  if (fd <0) {perror(DEVICE); exit(-1); }

  tcgetattr(fd,&oldtio);
  bzero(&newtio, sizeof(newtio));

  newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = IGNPAR | ICRNL;
  newtio.c_oflag = 0;
  newtio.c_lflag = ICANON;

  newtio.c_cc[VINTR]    = 0;     /* Ctrl-c */
  newtio.c_cc[VQUIT]    = 0;     /* Ctrl-\ */
  newtio.c_cc[VERASE]   = 0;     /* del */
  newtio.c_cc[VKILL]    = 0;     /* @ */
  newtio.c_cc[VEOF]     = 4;     /* Ctrl-d */
  newtio.c_cc[VTIME]    = 0;     /* inter-character timer unused */
  newtio.c_cc[VMIN]     = 1;     /* blocking read until 1 character arrives */
  newtio.c_cc[VSWTC]    = 0;     /* '\0' */
  newtio.c_cc[VSTART]   = 0;     /* Ctrl-q */
  newtio.c_cc[VSTOP]    = 0;     /* Ctrl-s */
  newtio.c_cc[VSUSP]    = 0;     /* Ctrl-z */
  newtio.c_cc[VEOL]     = 0;     /* '\0' */
  newtio.c_cc[VREPRINT] = 0;     /* Ctrl-r */
  newtio.c_cc[VDISCARD] = 0;     /* Ctrl-u */
  newtio.c_cc[VWERASE]  = 0;     /* Ctrl-w */
  newtio.c_cc[VLNEXT]   = 0;     /* Ctrl-v */
  newtio.c_cc[VEOL2]    = 0;     /* '\0' */

  fd_set rfds;
  struct timeval tv;

  while (STOP==FALSE) {     /* loop until we have a terminating condition */
    watchdog();
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);

    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);
    tv.tv_sec = 5;
    tv.tv_usec = 0;
    if ( select(fd+1, &rfds, NULL, NULL, &tv) < 1 ) {
    } else {
      res = read(fd,buf,255);
      buf[res]='\0';             /* set end of string, so we can printf */
      for (int i = 0; i <= res; i++)
        if ( (buf[i]=='\n') || (buf[i]=='\r') )
          buf[i]='\0';
      if ( checksumm(buf) ) STOP=TRUE;
    }
  }

  tcsetattr(fd,TCSANOW,&oldtio);
  close(fd);
  return ((string)buf);
}



bool checksumm(char *code) {
  //prueft Checksumme und schneidet sie ab.
  int count=0;
  int c=0, i;
  for (i=0; i<10; i++)
    if (isalpha(code[i]))
      count +=code[i] -'A' +10;
    else
      count +=code[i] -'0';
  count =count % 16;
  if (isalpha(code[10]))
    c +=code[10] -'A' +10;
  else
    c +=code[10] -'0';
  code[10]='\0';
  return ( c == count );
}


#endif

#ifdef CHIPDRIVE

unsigned short GetSmartcard (unsigned short);
string get_key();

string read_key() {
  string temp="";
  while ( temp=="" ) {
    watchdog();
    temp=get_key();
  }
  return ( temp );
}

string get_key () {
  unsigned short ctn;
  unsigned char read_binary[5] = { 0x00, 0xB0, 0x00, 0x00, 0x00 };
  unsigned char res[257];
  unsigned char dad;
  unsigned char sad;
  unsigned short lr;
  int i;
  char ret;
  char key[128];
  int address;
  int size;
  
  address=0;
  size=8;
  ctn=0;
  dad = 0;
  sad = 2;
  lr = 257;
  key[0]='\0';

  ret = CT_init (ctn, PORT - 1);
  if (ret != OK)
    {
      fprintf (stderr, "Error on port allocation: %d\n", ret);
      exit(1);
    }
  
  if (!GetSmartcard (ctn))
    CT_close (ctn);


  read_binary[2] = (unsigned char) (address >> 8);
  read_binary[3] = (unsigned char) (address & 0x00FF);
  read_binary[4] = (unsigned char) size;

  ret = CT_data (ctn, &dad, &sad, 5, read_binary, &lr, res);
  if ( ret != OK ) { return (""); }

  for (i = 0; i < lr -2; i++)
    sprintf (key,"%s%02X", key, res[i]);

  CT_close (ctn);
  return ( (string) key);
}


unsigned short GetSmartcard (unsigned short ctn) {
  unsigned char cmd[5], res[256], sad, dad;
  unsigned short lr, count;
  char ret;

  for (count = 0; count < 10; count ++)
    {
      cmd[0] = CTBCS_CLA;
      cmd[1] = CTBCS_INS_STATUS;
      cmd[2] = CTBCS_P1_CT_KERNEL;
      cmd[3] = CTBCS_P2_STATUS_ICC;
      cmd[4] = 0x00;

      dad = 1;
      sad = 2;
      lr = 256;

      ret = CT_data (ctn, &dad, &sad, 5, cmd, &lr, res);

      if ((ret != OK) || (res[lr-2] != 0x90)) { return 0; }
 
      if (res[0] == CTBCS_DATA_STATUS_CARD_CONNECT)
        {            
          cmd[0] = CTBCS_CLA;
          cmd[1] = CTBCS_INS_REQUEST;
          cmd[2] = CTBCS_P1_INTERFACE1;
          cmd[3] = CTBCS_P2_REQUEST_GET_ATR;
          cmd[4] = 0x00;

          dad = 0x01;
          sad = 0x02;
          lr = 256;

          ret = CT_data (ctn, &dad, &sad, 5, cmd, &lr, res);

          if ((ret != OK) || (res[lr - 2] != 0x90)) { return 0; }
          return 1;
        }
        usleep (999999);
    }
  return 0;
}


#endif

    
void watchdog() {
  ofstream wd (WATCHDOG);
  static int count=0;
  wd << count << endl;
  count++;
  wd.close();
}

